# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-12-16 23:22
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0004_eveapikeypair_sso_verified'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='eveallianceinfo',
            name='member_count',
        ),
    ]
