{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load staticfiles %}
{% load i18n %}

{% block page_title %}
{% endblock page_title %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">
            {% block page_header %}
            {% endblock %}
            {% include 'timerboard/index_button.html' %}
        </h1>
        <div class="container-fluid">
            <div class="col-md-4 col-md-offset-4">
                <div class="row">
                    <form id="add-timer-form" class="form-signin" role="form" action="" method="POST">
                        {% csrf_token %}
                        {{ form|bootstrap }}
                        <br/>
                        <button class="btn btn-lg btn-primary btn-block" type="submit">
                            {% block submit_button_text %}
                            {% endblock %}
                        </button>
                    </form>
                </div>
            </div>
        </div>
    </div>

{% endblock content %}
