# Generated by Django 2.0.6 on 2018-08-03 04:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fleetactivitytracking', '0005_remove_fat_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fat',
            name='shiptype',
            field=models.CharField(max_length=100),
        ),
    ]
