# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-12 01:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    atomic = False

    dependencies = [
        ('teamspeak3', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='authts',
            table=None,
        ),
        migrations.AlterModelTable(
            name='tsgroup',
            table=None,
        ),
        migrations.AlterModelTable(
            name='usertsgroup',
            table=None,
        ),
    ]
