{% extends "allianceauth/base.html" %}
{% load bootstrap %}
{% load staticfiles %}
{% load i18n %}

{% block page_title %}{% trans "Fatlink statistics" %}{% endblock page_title %}

{% block content %}
    <div class="col-lg-12">
        <h1 class="page-header text-center">{% blocktrans %}Participation data statistics for {{ month }}, {{ year }}{% endblocktrans %}
            <div class="text-right">
                <a href="{% url 'fatlink:statistics_month' previous_month|date:"Y" previous_month|date:"m"  %}" class="btn btn-info">{% trans "Previous month" %}</a>
                {%  if next_month %}
                <a href="{% url 'fatlink:statistics_month' next_month|date:"Y" next_month|date:"m"  %}" class="btn btn-info">{% trans "Next month" %}</a>
                {%  endif %}
            </div>
        </h1>
        {% if fatStats %}
        <table class="table table-responsive">
            <tr>
                <th class="col-md-1"></th>
                <th class="col-md-2 text-center">{% trans "Ticker" %}</th>
                <th class="col-md-5 text-center">{% trans "Corp" %}</th>
                <th class="col-md-2 text-center">{% trans "Members" %}</th>
                <th class="col-md-2 text-center">{% trans "Fats" %}</th>
                <th class="col-md-2 text-center">{% trans "Average fats" %}
                  <i class="glyphicon glyphicon-question-sign" rel="tooltip" title="Fats ÷ Characters"></i>
                </th>
            </tr>
            {% for corpStat in fatStats %}
            <tr>
                <td>
                    <img src="{{ corpStat.corp.logo_url_32 }}" class="ra-avatar img-responsive">
                </td>
                <td class="text-center"><a href="{% url 'fatlink:statistics_corp' corpStat.corp.corporation_id %}">[{{ corpStat.corp.corporation_ticker }}]</td>
                <td class="text-center">{{ corpStat.corp.corporation_name }}</td>
                <td class="text-center">{{ corpStat.corp.member_count }}</td>
                <td class="text-center">{{ corpStat.n_fats }}</td>
                <td class="text-center">{{ corpStat.avg_fat }}</td>
            </tr>
            {% endfor %}
        </table>
        {% endif %}
    </div>
{% endblock content %}
{% block extra_script %}
$(document).ready(function(){
  $("[rel=tooltip]").tooltip();
{% endblock extra_script %}
