.gray-icon-color .fa {
    color: #505050;
}

.notification-bell-color {
    color: #A88F1E;
}

.navbar-brand {
    height: 58px;
    padding: 19px 19px;
}

.auth-navbar-top {
    margin-bottom: 0;
}

.auth-side-navbar-collapse {
    padding-left: 0;
}

#wrapper {
    width: 100%;
    padding: 0;
}

#site-body-wrapper {
    margin-right:0;
}

/* Horizontal list group */
ul.list-group.list-group-horizontal {
    display: table;
    width: 100%;
}

ul.list-group.list-group-horizontal > li.list-group-item {
    display: table-cell;
    max-width: 0;
}

.flex-center-horizontal {
    justify-content: center;
}
