{% load i18n %}

<tr>
    <td class="text-center">Teamspeak 3</td>
    <td class="text-center">{{ authinfo.teamspeak3_uid }}</td>
    <td class="text-center"><a href="ts3server://{{ TEAMSPEAK3_PUBLIC_URL }}">{{ TEAMSPEAK3_PUBLIC_URL }}</a></td>
    <td class="text-center">
        {% ifequal authinfo.teamspeak3_uid "" %}
            <a href="{% url 'teamspeak3:activate' %}" title="Activate" class="btn btn-warning">
                <span class="glyphicon glyphicon-ok"></span>
            </a>
        {% else %}
            <a href="{% url 'teamspeak3:verify' %}" title="Verify Client ID" class="btn btn-success" title="Verify">
                <span class="glyphicon glyphicon-log-in"></span>
            </a>
            <a href="{% url 'teamspeak3:reset_perm' %}" title="Refresh Token" class="btn btn-primary">
                <span class="glyphicon glyphicon-refresh"></span>
            </a>
            <a href="{% url 'teamspeak3:deactivate' %}" title="Deactivate" class="btn btn-danger">
                <span class="glyphicon glyphicon-remove"></span>
            </a>
            <a href="ts3server://{{ TEAMSPEAK3_PUBLIC_URL }}?nickname={{ authinfo.teamspeak3_uid }}" title="Connect" class="btn btn-success">
                <span class="glyphicon glyphicon-arrow-right"></span>
            </a>
        {% endifequal %}
    </td>
</tr>
