# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-11-01 04:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0003_auto_20161026_0149'),
    ]

    operations = [
        migrations.AddField(
            model_name='eveapikeypair',
            name='sso_verified',
            field=models.BooleanField(default=False),
        ),
    ]
