{% load i18n %}
{% load navactive %}
{% load menu_items %}
<div class="col-sm-2 auth-side-navbar" role="navigation">
    <div class="collapse navbar-collapse auth-menus-collapse auth-side-navbar-collapse">
        <ul class="nav nav-pills nav-stacked gray-icon-color" id="side-menu">
            <li>
                <a class="{% navactive request 'authentication:dashboard' %}"
                   href="{% url 'authentication:dashboard' %}">
                    <i class="fa fa-dashboard fa-fw"></i> {% trans "Dashboard" %}
                </a>
            </li>
            <li>
                <a class="{% navactive request 'groupmanagement:groups' %}" href="{% url 'groupmanagement:groups' %}">
                    <i class="fa fa-cogs fa-fw fa-sitemap"></i> {% trans "Groups" %}
                </a>
            </li>

            {% if can_manage_groups %}
                <li>
                    <a class="{% navactive request 'groupmanagement:management groupmanagement:membership groupmanagement:membership_list' %}"
                       href="{% url 'groupmanagement:management' %}">
                        <i class="fa fa-lock fa-sitemap fa-fw"></i> {% trans "Group Management" %}
                    </a>
                </li>
            {% endif %}

            {% menu_items %}

            {% if user.is_superuser %}
                <li>
                    <a class="{% navactive request 'authentication:help' %}"
                       href="{% url 'authentication:help' %}">
                        <i class="fa fa-question fa-fw"></i> {% trans "Help" %}
                    </a>
                </li>
            {% endif %}
        </ul>
    </div>
</div>
